package com.zanxiang.game.module.manage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.game.module.manage.pojo.params.UserRechargeWhiteAddParam;
import com.zanxiang.game.module.manage.pojo.params.UserRechargeWhiteListParam;
import com.zanxiang.game.module.manage.pojo.vo.UserRechargeWhiteVO;
import com.zanxiang.game.module.manage.service.IUserRechargeWhiteService;
import com.zanxiang.module.util.pojo.ResultVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2025-05-13
 * @description :
 */
@Api(tags = {"玩家充值白名单接口"})
@RestController
@RequestMapping("/user/recharge/white")
@Slf4j
public class UserRechargeWhiteController {

    @Autowired
    private IUserRechargeWhiteService userRechargeWhiteService;

    @ApiOperation(value = "查询有配置的游戏")
    @GetMapping(value = "/game/list")
    @PreAuthorize(permissionKey = "manage:userRechargeWhite:gameList")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Map.class)})
    public ResultVO<Map<Long, String>> chatGameMap() {
        return ResultVO.ok(userRechargeWhiteService.rechargeWhiteGameMap());
    }

    @ApiOperation(value = "添加白名单配置")
    @PutMapping(value = "/add")
    @PreAuthorize(permissionKey = "manage:userRechargeWhite:add")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> addRechargeWhite(@RequestBody @Validated UserRechargeWhiteAddParam param) {
        return ResultVO.ok(userRechargeWhiteService.addRechargeWhite(param));
    }

    @ApiOperation(value = "查询白名单配置列表")
    @PostMapping(value = "/list")
    @PreAuthorize(permissionKey = "manage:userRechargeWhite:listOfPage")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = UserRechargeWhiteVO.class)})
    public ResultVO<IPage<UserRechargeWhiteVO>> listOfPage(@Validated @RequestBody UserRechargeWhiteListParam param) {
        return ResultVO.ok(userRechargeWhiteService.listOfPage(param));
    }

    @ApiOperation(value = "白名单删除配置")
    @DeleteMapping(value = "/del")
    @PreAuthorize(permissionKey = "manage:userRechargeWhite:del")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> delRechargeWhite(@RequestParam Long id) {
        return ResultVO.ok(userRechargeWhiteService.delRechargeWhite(id));
    }
}
